/*
Definitions of common runtime errors. The word 'error' in the sense that it is
used here can refer equally to recoverable and non-recoverable faults.
*/

/*
TODO: Experiment with dumping state using errors.
TODO: Use supplants whereever possible.
*/

define(function () {
    var _ = {};        // Public namespace.
    var __ = {};       // Private namespace.
    /*
    An error which can be "deflated" into json or printed as a string. This
    representation can saved, transferred between workers, or sent accross the
    network.
    */
    _.SerializableError = function () {
        this.abstract = true;
        this.prototype.name = null;
        this.prototype.message = null;
    };
    _.SerializableError.fn = _.SerializableError.prototype = new Error();
    _.SerializableError.constructor = _.SerializableError;
    /*
    Constructs a JSON representation of the Error.
    */
    _.SerializableError.fn.toJSON = function () {
        var result = this.toString();
        return "{" + result + "}";
    };
    /*
    Constructs a string representation of the Error.
    */
    _.SerializableError.fn.toString = function () {
        var result = "name: {name}, message: {message}";
        result = is.string.supplant(result)({name: this.name, message: this.message});
        return result;
    };
    /*
    Manufactures a SerializableError.
    */
    _.ErrorFactory = function (name, defualtMessage) {
        var tmp = function (message) {
            this.name = name;
            this.message = message || defualtMessage;
        };
        tmp.prototype = new SerializableError();
        tmp.constructor = tmp;
        return tmp;
    };
    /*
    Thrown when a function is called which has not been implemented.
    */
    _.NotImplementedError = _.ErrorFactory("NotImplementedError", "The function is not implemented.");
    /*
    Thrown when a value is not of the type which is required for a routine to continue
    execution.
    */
    _.ArgumentError = _.ErrorFactory("ArgumentError", "An argument is invalid.");
    /*
    Thrown when a routine cannot continue because a value does not satisfy a required
    type.
    */
    _.TypeError = _.ErrorFactory("TypeError", "A value does not satisfy a required type.");
    /*
    Thrown when a predicate which is assumed to be true at a point in program execution
    evaluates to false.
    */
    _.AssertionError = _.ErrorFactory("AssertionError", "The assertion is invalid.");
    /*
    Thrown when an object cannot be initialized due to a failure in securing network /
    system / application resources.
    */
    _.InitializationError = _.ErrorFactory("InitializationError", "The object could not be initalized.");
    /*
    Thrown when an object cannot be instantiated because it is a singleton, abstract,
    or defines an interface.
    */
    _.InstantiationError = _.ErrorFactory("InstantiationError", "The object could not be instantiated.");
    /*
    Thrown when a routine cannot continue because it requires a value to defined and non-null.
    */
    _.NullValueError = _.ErrorFactory("NullValueError", "The value was null or undefined.");
});
