/*
Logging provides an extensible logging facility which supports specifying multiple
destinations and grouping loggers together.
*/
define(['/libs/wyvern/util/introspect.js',
        '/libs/wyvern/util/errors.js'],
    function (is, er) {
    var _ = {};        // Public namespace.
    var __ = {};       // Private namespace.
    /*
    Debugging strategies which can provide various levels of detail about the origin
    of logger messages.
    */
    __.debugHandlers = Object.freeze({
        "source": function (msg) {
            is.assertArgumentIs(msg, "string", "msg");
            console.warn(this.arguments.caller.caller.name); 
        },
        "stacktrace": function (msg) {
            /*
            TODO: Use the native code stracktrace made available by the browser.
            */
            is.assertArgumentIs(msg, "string", "msg");
            var maxDepth = 15;
            var stackframe = this.arguments.caller.caller;
            console.group(msg);
            while (stackframe && maxDepth--) {
                console.warn(stackframe.caller.name);
                stackframe = stackframe.caller;
            }
            console.groupEnd();
        }
    });
    /*
    Valid debugging modes.
    */
    __.debugModes = Object.freeze(__.debugHandlers.keys());
    /*
    Checks if a an object is a logger which has been initialized properly.
    */
    __.isValidLogger = function (object) {
        return object instanceof _.Logger &&
            is.typeOf(object.name) === "string" &&
            is.typeOf(object.debug_mode) === "string" &&
            is.assert(object.debug_mode in __.debugModes);
    };
    /*
    Asserts that a logger is valid, otherwise throws an InitializationError.
    */
    __.assertIsValidLogger = function (object) {
        is.assert(__.isValidLogger(object), "The logger was not properly initialized.");
    };
    /*
    Instantiates an empty Logger.
    */
    _.Logger = function () {
        this.name = null;
        this.propogate = null;
        this.debug_mode = null;
        this.sinks = null;
    };
    _.Logger.fn = _.Logger.prototype;
    /*
    Initializes the Logger -- setting the name and mode.
    */
    _.Logger.fn.init = function (name, debug_mode, propogate) {
        is.assertArgumentsAre(arguments, {
            "name": {type: "string", required: true},
            "debug_mode": {type: "string"},
            "propogate": {type: "boolean"}
        });
        this.name = name;
        this.propogate = propogate;
        this.debugMode = debugMode;
        this.sinks = {};
    };
    /*
    Appends a message to the log.
    */
    _.Logger.fn.log = function (msg) {
        __.assertIsValidLogger(this);
        is.assertArgumentIs(msg, "string", "msg");
        if (this.debugMode) {
            __.debugHandlers[this.debugMode]();
        }
    };
    /*
    Registers a sink for the logger.
    */
    _.Logger.fn.registerSink = function (name, uri, handler) {
        throw new er.NotImplementedError("registerSink has not yet been implemented.");
        __.assertIsValidLogger(this);
        is.assertArgumentsAre(arguments, {
            "name": {type: "string", required: true},
            "options": {type: "options", required: true},
            "handler": {type: "string", required: true}
        });
    };
    return _;
});
